@@ -28,6 +28,12 @@ extern "C" {
 
   void rpg_devroye(double *x, int *n, double *z, int *num);
 
+  void rpg_alt(double *x, double *h, double *z, int* num);
+
+  void rpg_sp(double *x, double *h, double *z, int* num, int* iter);
+
+  void rpg_hybrid(double *x, double *h, double *z, int* num);
+
   // Default Logistic
 
   void gibbs(double *wp, double *betap,                            // Posterior
